searchData={"items":[{"type":"module","title":"BTHome.ButtonEvent","doc":"Defines button events for BTHome v2 format.\n\nThis module provides constants for button events that match the Python\nBTHome library's ButtonEvent enum values.","ref":"BTHome.ButtonEvent.html"},{"type":"function","title":"BTHome.ButtonEvent.all/0","doc":"Returns all valid button event types.","ref":"BTHome.ButtonEvent.html#all/0"},{"type":"function","title":"BTHome.ButtonEvent.double_press/0","doc":"Double button press event.","ref":"BTHome.ButtonEvent.html#double_press/0"},{"type":"function","title":"BTHome.ButtonEvent.hold_press/0","doc":"Button hold press event.","ref":"BTHome.ButtonEvent.html#hold_press/0"},{"type":"function","title":"BTHome.ButtonEvent.long_double_press/0","doc":"Long double button press event.","ref":"BTHome.ButtonEvent.html#long_double_press/0"},{"type":"function","title":"BTHome.ButtonEvent.long_press/0","doc":"Long button press event.","ref":"BTHome.ButtonEvent.html#long_press/0"},{"type":"function","title":"BTHome.ButtonEvent.long_triple_press/0","doc":"Long triple button press event.","ref":"BTHome.ButtonEvent.html#long_triple_press/0"},{"type":"function","title":"BTHome.ButtonEvent.none/0","doc":"No button event occurred.","ref":"BTHome.ButtonEvent.html#none/0"},{"type":"function","title":"BTHome.ButtonEvent.press/0","doc":"Single button press event.","ref":"BTHome.ButtonEvent.html#press/0"},{"type":"function","title":"BTHome.ButtonEvent.release/0","doc":"Button release event.","ref":"BTHome.ButtonEvent.html#release/0"},{"type":"function","title":"BTHome.ButtonEvent.triple_press/0","doc":"Triple button press event.","ref":"BTHome.ButtonEvent.html#triple_press/0"},{"type":"function","title":"BTHome.ButtonEvent.valid?/1","doc":"Checks if the given value is a valid button event type.","ref":"BTHome.ButtonEvent.html#valid?/1"},{"type":"module","title":"BTHome.DimmerEvent","doc":"Defines dimmer events for BTHome v2 format.\n\nThis module provides constants for dimmer events that match the Python\nBTHome library's DimmerEvent enum values.","ref":"BTHome.DimmerEvent.html"},{"type":"function","title":"BTHome.DimmerEvent.all/0","doc":"Returns all valid dimmer event types.","ref":"BTHome.DimmerEvent.html#all/0"},{"type":"function","title":"BTHome.DimmerEvent.none/0","doc":"No dimmer event occurred.","ref":"BTHome.DimmerEvent.html#none/0"},{"type":"function","title":"BTHome.DimmerEvent.rotate_left/0","doc":"Dimmer rotated left.","ref":"BTHome.DimmerEvent.html#rotate_left/0"},{"type":"function","title":"BTHome.DimmerEvent.rotate_right/0","doc":"Dimmer rotated right.","ref":"BTHome.DimmerEvent.html#rotate_right/0"},{"type":"function","title":"BTHome.DimmerEvent.valid?/1","doc":"Checks if the given value is a valid dimmer event type.","ref":"BTHome.DimmerEvent.html#valid?/1"},{"type":"module","title":"BTHome.Encryption","doc":"AES-CCM encryption/decryption for BTHome v2 protocol.\n\nImplements the BTHome v2 encryption specification using AES-CCM mode\nwith 16-byte keys and 4-byte MIC (Message Integrity Check).","ref":"BTHome.Encryption.html"},{"type":"module","title":"Key Requirements - BTHome.Encryption","doc":"- Encryption key must be exactly 16 bytes (128-bit)\n- MAC address must be exactly 6 bytes\n- Counter is a 4-byte unsigned integer (little-endian)\n- MIC (Message Integrity Check) is always 4 bytes","ref":"BTHome.Encryption.html#module-key-requirements"},{"type":"module","title":"Nonce Construction - BTHome.Encryption","doc":"The nonce is constructed by concatenating:\n1. MAC address (6 bytes)\n2. BTHome UUID (2 bytes, always 0xD2FC)\n3. Device info byte (1 byte)\n4. Counter (4 bytes, little-endian)\n\nTotal nonce length: 13 bytes","ref":"BTHome.Encryption.html#module-nonce-construction"},{"type":"module","title":"Examples - BTHome.Encryption","doc":"# Encrypt sensor data\n    key = <<0x23, 0x1d, 0x39, 0xc1, 0xd7, 0xcc, 0x1a, 0xb1,\n            0xae, 0xe2, 0x24, 0xcd, 0x09, 0x6d, 0xb9, 0x32>>\n    mac = <<0x54, 0x48, 0xe6, 0x8f, 0x80, 0xa5>>\n    data = <<0x02, 0xca, 0x09, 0x03, 0xbf, 0x13>>  # temp + humidity\n    counter = 0x33221100\n\n    {:ok, {ciphertext, mic}} = BTHome.Encryption.encrypt(data, key, mac, counter)\n\n    # Decrypt sensor data\n    {:ok, decrypted} = BTHome.Encryption.decrypt(ciphertext, mic, key, mac, counter)","ref":"BTHome.Encryption.html#module-examples"},{"type":"function","title":"BTHome.Encryption.decrypt/5","doc":"Decrypts BTHome sensor data using AES-CCM.","ref":"BTHome.Encryption.html#decrypt/5"},{"type":"function","title":"Parameters - BTHome.Encryption.decrypt/5","doc":"- `ciphertext` - Encrypted sensor data\n- `mic` - 4-byte Message Integrity Check\n- `key` - 16-byte encryption key (same as used for encryption)\n- `mac_address` - 6-byte MAC address of the device\n- `counter` - 4-byte counter value used during encryption\n- `device_info` - Device information byte (optional, defaults to encrypted v2)","ref":"BTHome.Encryption.html#decrypt/5-parameters"},{"type":"function","title":"Returns - BTHome.Encryption.decrypt/5","doc":"- `{:ok, data}` - Successfully decrypted sensor data\n- `{:error, reason}` - Decryption failed (wrong key, tampered data, etc.)","ref":"BTHome.Encryption.html#decrypt/5-returns"},{"type":"function","title":"Examples - BTHome.Encryption.decrypt/5","doc":"{:ok, data} = decrypt(ciphertext, mic, key, mac, 1)","ref":"BTHome.Encryption.html#decrypt/5-examples"},{"type":"function","title":"BTHome.Encryption.encrypt/4","doc":"Encrypts BTHome sensor data using AES-CCM.","ref":"BTHome.Encryption.html#encrypt/4"},{"type":"function","title":"Parameters - BTHome.Encryption.encrypt/4","doc":"- `data` - Binary sensor data to encrypt\n- `key` - 16-byte encryption key\n- `mac_address` - 6-byte MAC address of the device\n- `counter` - 4-byte counter value (unsigned integer)\n- `device_info` - Device information byte (optional, defaults to encrypted v2)","ref":"BTHome.Encryption.html#encrypt/4-parameters"},{"type":"function","title":"Returns - BTHome.Encryption.encrypt/4","doc":"- `{:ok, {ciphertext, mic}}` - Success with encrypted data and MIC\n- `{:error, reason}` - Encryption failed","ref":"BTHome.Encryption.html#encrypt/4-returns"},{"type":"function","title":"Examples - BTHome.Encryption.encrypt/4","doc":"key = :crypto.strong_rand_bytes(16)\n    mac = <<0x54, 0x48, 0xe6, 0x8f, 0x80, 0xa5>>\n    data = <<0x02, 0xca, 0x09>>  # temperature measurement\n\n    {:ok, {ciphertext, mic}} = encrypt(data, key, mac, 1)","ref":"BTHome.Encryption.html#encrypt/4-examples"},{"type":"function","title":"BTHome.Encryption.generate_key/0","doc":"Generates a random encryption key.","ref":"BTHome.Encryption.html#generate_key/0"},{"type":"function","title":"Returns - BTHome.Encryption.generate_key/0","doc":"16-byte random encryption key","ref":"BTHome.Encryption.html#generate_key/0-returns"},{"type":"function","title":"BTHome.Encryption.key_from_hex/1","doc":"Converts a hex string to binary key.","ref":"BTHome.Encryption.html#key_from_hex/1"},{"type":"function","title":"Parameters - BTHome.Encryption.key_from_hex/1","doc":"- `hex_string` - Hex string representation of key (32 characters)","ref":"BTHome.Encryption.html#key_from_hex/1-parameters"},{"type":"function","title":"Returns - BTHome.Encryption.key_from_hex/1","doc":"- `{:ok, key}` - Successfully converted key\n- `{:error, reason}` - Invalid hex string","ref":"BTHome.Encryption.html#key_from_hex/1-returns"},{"type":"function","title":"Examples - BTHome.Encryption.key_from_hex/1","doc":"{:ok, key} = key_from_hex(\"231d39c1d7cc1ab1aee224cd096db932\")","ref":"BTHome.Encryption.html#key_from_hex/1-examples"},{"type":"function","title":"BTHome.Encryption.key_to_hex/1","doc":"Converts a binary key to hex string.","ref":"BTHome.Encryption.html#key_to_hex/1"},{"type":"function","title":"Parameters - BTHome.Encryption.key_to_hex/1","doc":"- `key` - Binary encryption key","ref":"BTHome.Encryption.html#key_to_hex/1-parameters"},{"type":"function","title":"Returns - BTHome.Encryption.key_to_hex/1","doc":"Hex string representation of the key","ref":"BTHome.Encryption.html#key_to_hex/1-returns"},{"type":"function","title":"Examples - BTHome.Encryption.key_to_hex/1","doc":"hex = key_to_hex(key)  # => \"231d39c1d7cc1ab1aee224cd096db932\"","ref":"BTHome.Encryption.html#key_to_hex/1-examples"},{"type":"module","title":"BTHome.Packet","doc":"Builder pattern for creating BTHome v2 packets with a pipeable API.\n\nThis module provides a fluent interface for building BTHome v2 packets by\naccumulating measurements and then serializing them. It supports the builder\npattern with method chaining using the pipe operator.","ref":"BTHome.Packet.html"},{"type":"module","title":"Examples - BTHome.Packet","doc":"# Basic usage with builder pattern\n    {:ok, binary} = BTHome.new_packet()\n    |> BTHome.add_measurement(:temperature, 23.45)\n    |> BTHome.add_measurement(:motion, true)\n    |> BTHome.serialize()\n\n    # With encryption\n    {:ok, binary} = BTHome.new_packet()\n    |> BTHome.add_measurement(:temperature, 23.45)\n    |> BTHome.add_measurement(:humidity, 67.8)\n    |> BTHome.serialize(true)\n\n    # Error handling in the chain\n    result = BTHome.new_packet()\n    |> BTHome.add_measurement(:temperature, 23.45)\n    |> BTHome.add_measurement(:invalid_type, 42)  # This will return an error\n    |> BTHome.serialize()\n\n    case result do\n      {:ok, binary} -> handle_success(binary)\n      {:error, error} -> handle_error(error)\n    end","ref":"BTHome.Packet.html#module-examples"},{"type":"function","title":"BTHome.Packet.add_measurement/4","doc":"Adds a measurement to the packet builder.\n\nIf the packet already contains an error from a previous operation, this\nfunction will return the packet unchanged. Otherwise, it validates the\nmeasurement and adds it to the packet if valid.","ref":"BTHome.Packet.html#add_measurement/4"},{"type":"function","title":"Parameters - BTHome.Packet.add_measurement/4","doc":"- `packet` - The packet builder struct\n- `type` - The measurement type atom\n- `value` - The measurement value\n- `opts` - Optional keyword list (same as `BTHome.Measurement.new/3`)","ref":"BTHome.Packet.html#add_measurement/4-parameters"},{"type":"function","title":"Returns - BTHome.Packet.add_measurement/4","doc":"The updated packet builder struct, potentially with an error if validation fails.","ref":"BTHome.Packet.html#add_measurement/4-returns"},{"type":"function","title":"Examples - BTHome.Packet.add_measurement/4","doc":"iex> packet = BTHome.new_packet()\n    iex> result = BTHome.add_measurement(packet, :temperature, 23.45)\n    iex> length(result.measurements)\n    1\n    iex> is_nil(result.error)\n    true\n\n    iex> packet = BTHome.new_packet()\n    iex> result = BTHome.add_measurement(packet, :invalid_type, 42)\n    iex> match?(%BTHome.Packet{measurements: [], error: \"Unsupported measurement type: :invalid_type\"}, result)\n    true","ref":"BTHome.Packet.html#add_measurement/4-examples"},{"type":"function","title":"BTHome.Packet.new/0","doc":"Creates a new empty packet builder.\n\nReturns a new `BTHome.Packet` struct that can be used to accumulate\nmeasurements using the builder pattern.","ref":"BTHome.Packet.html#new/0"},{"type":"function","title":"Returns - BTHome.Packet.new/0","doc":"A new packet builder struct.","ref":"BTHome.Packet.html#new/0-returns"},{"type":"function","title":"Examples - BTHome.Packet.new/0","doc":"iex> BTHome.new_packet()\n    %BTHome.Packet{measurements: [], error: nil}","ref":"BTHome.Packet.html#new/0-examples"},{"type":"function","title":"BTHome.Packet.serialize/2","doc":"Serializes the accumulated measurements in the packet.","ref":"BTHome.Packet.html#serialize/2"},{"type":"function","title":"Parameters - BTHome.Packet.serialize/2","doc":"- `packet` - The packet builder struct\n- `opts` - Serialization options (keyword list)\n  - `:encrypt` - Encryption options (keyword list or map)\n    - `:key` - 16-byte encryption key (required)\n    - `:mac_address` - 6-byte MAC address (required)\n    - `:counter` - 4-byte counter value (required)","ref":"BTHome.Packet.html#serialize/2-parameters"},{"type":"function","title":"Returns - BTHome.Packet.serialize/2","doc":"`{:ok, binary}` on success, `{:error, reason}` on failure.","ref":"BTHome.Packet.html#serialize/2-returns"},{"type":"function","title":"Examples - BTHome.Packet.serialize/2","doc":"# Basic usage\n    iex> {:ok, binary} = BTHome.new_packet()\n    iex> |> BTHome.add_measurement(:temperature, 23.45)\n    iex> |> BTHome.serialize()\n    iex> is_binary(binary)\n    true\n\n    # With encryption\n    iex> key = <<1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16>>\n    iex> mac = <<0xAA, 0xBB, 0xCC, 0xDD, 0xEE, 0xFF>>\n    iex> opts = [encrypt: [key: key, mac_address: mac, counter: 1]]\n    iex> {:ok, binary} = BTHome.new_packet()\n    iex> |> BTHome.add_measurement(:temperature, 23.45)\n    iex> |> BTHome.serialize(opts)\n    iex> is_binary(binary)\n    true","ref":"BTHome.Packet.html#serialize/2-examples"},{"type":"function","title":"BTHome.Packet.serialize!/2","doc":"Serializes the accumulated measurements in the packet, raising on error.\n\nSimilar to `serialize/2` but raises an exception instead of returning an error tuple.\nThis is useful for pipeline operations where you want to fail fast on errors.","ref":"BTHome.Packet.html#serialize!/2"},{"type":"function","title":"Parameters - BTHome.Packet.serialize!/2","doc":"- `packet` - The packet builder struct\n- `opts` - Serialization options (same as `serialize/2`)","ref":"BTHome.Packet.html#serialize!/2-parameters"},{"type":"function","title":"Returns - BTHome.Packet.serialize!/2","doc":"Binary BTHome v2 data on success, raises on error.","ref":"BTHome.Packet.html#serialize!/2-returns"},{"type":"function","title":"Examples - BTHome.Packet.serialize!/2","doc":"# Basic usage\n    iex> binary = BTHome.new_packet()\n    iex> |> BTHome.add_measurement(:temperature, 23.45)\n    iex> |> BTHome.serialize!()\n    iex> is_binary(binary)\n    true\n\n    # With encryption\n    iex> key = <<1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16>>\n    iex> mac = <<0xAA, 0xBB, 0xCC, 0xDD, 0xEE, 0xFF>>\n    iex> opts = [encrypt: [key: key, mac_address: mac, counter: 1]]\n    iex> binary = BTHome.new_packet()\n    iex> |> BTHome.add_measurement(:temperature, 23.45)\n    iex> |> BTHome.serialize!(opts)\n    iex> is_binary(binary)\n    true","ref":"BTHome.Packet.html#serialize!/2-examples"},{"type":"function","title":"Raises - BTHome.Packet.serialize!/2","doc":"`ArgumentError` if serialization fails or packet contains an error.","ref":"BTHome.Packet.html#serialize!/2-raises"},{"type":"type","title":"BTHome.Packet.t/0","doc":"","ref":"BTHome.Packet.html#t:t/0"},{"type":"module","title":"BTHome","doc":"BTHome v2 main API module.\n\nThis module provides the primary interface for working with BTHome v2 sensor data.\nIt handles serialization to/from binary format, validation, and measurement creation\nwith full type safety and error handling.","ref":"BTHome.html"},{"type":"module","title":"Features - BTHome","doc":"- **Type Safety**: Uses structs for measurements and decoded data\n- **Validation**: Comprehensive validation of measurement types and values\n- **Error Handling**: Structured errors with context information\n- **Performance**: Compile-time optimizations for fast lookups\n- **Compatibility**: Struct-based input with map output for deserialization","ref":"BTHome.html#module-features"},{"type":"module","title":"Supported Sensor Types - BTHome","doc":"","ref":"BTHome.html#module-supported-sensor-types"},{"type":"module","title":"Environmental Sensors - BTHome","doc":"- `:temperature` - Temperature in °C\n- `:humidity` - Relative humidity in %\n- `:pressure` - Atmospheric pressure in hPa\n- `:illuminance` - Light level in lux\n- `:battery` - Battery level in %\n- `:energy` - Energy consumption in kWh\n- `:power` - Power consumption in W\n- `:voltage` - Voltage in V","ref":"BTHome.html#module-environmental-sensors"},{"type":"module","title":"Binary Sensors - BTHome","doc":"- `:motion` - Motion detection (true/false)\n- `:door` - Door state (open/closed)\n- `:window` - Window state (open/closed)\n- `:occupancy` - Room occupancy (occupied/vacant)\n- `:smoke` - Smoke detection (detected/clear)\n- `:battery_low` - Low battery warning (low/ok)","ref":"BTHome.html#module-binary-sensors"},{"type":"module","title":"Basic Usage - BTHome","doc":"# Create measurements using the struct API\n    {:ok, temp} = BTHome.measurement(:temperature, 23.45)\n    {:ok, motion} = BTHome.measurement(:motion, true)\n\n    # Serialize to binary\n    {:ok, binary} = BTHome.serialize([temp, motion])\n\n    # Deserialize back to structs\n    {:ok, decoded} = BTHome.deserialize(binary)","ref":"BTHome.html#module-basic-usage"},{"type":"module","title":"Deserialization Output - BTHome","doc":"# Deserialization returns convenient map format\n    {:ok, binary} = BTHome.serialize([temp, motion])\n    {:ok, %{measurements: measurements}} = BTHome.deserialize(binary)\n    # measurements is a list of maps for easy access","ref":"BTHome.html#module-deserialization-output"},{"type":"module","title":"Validation - BTHome","doc":"# Validate before serialization\n    case BTHome.validate_measurements(measurements) do\n      :ok -> BTHome.serialize(measurements)\n      {:error, reason} -> handle_error(reason)\n    end","ref":"BTHome.html#module-validation"},{"type":"module","title":"Error Handling - BTHome","doc":"All functions return tagged tuples with structured errors:\n\n    {:error, %BTHome.Error{\n      type: :invalid_data,\n      message: \"Unsupported measurement type: :invalid\",\n      context: %{type: :invalid}\n    }}","ref":"BTHome.html#module-error-handling"},{"type":"function","title":"BTHome.add_measurement/4","doc":"Adds a measurement to a packet builder.\n\nThis function is designed to work with the builder pattern, allowing you to\nchain multiple measurements using the pipe operator. If any measurement fails\nvalidation, the error is captured and will be returned when `serialize/1` is called.","ref":"BTHome.html#add_measurement/4"},{"type":"function","title":"Parameters - BTHome.add_measurement/4","doc":"- `packet` - The packet builder struct (from `new_packet/0` or previous `add_measurement/4`)\n- `type` - The measurement type atom\n- `value` - The measurement value\n- `opts` - Optional keyword list (same options as `measurement/3`)","ref":"BTHome.html#add_measurement/4-parameters"},{"type":"function","title":"Returns - BTHome.add_measurement/4","doc":"The updated packet builder struct.","ref":"BTHome.html#add_measurement/4-returns"},{"type":"function","title":"Examples - BTHome.add_measurement/4","doc":"# Single measurement\n    packet = BTHome.new_packet()\n    |> BTHome.add_measurement(:temperature, 23.45)\n\n    # Multiple measurements\n    packet = BTHome.new_packet()\n    |> BTHome.add_measurement(:temperature, 23.45)\n    |> BTHome.add_measurement(:humidity, 67.8)\n    |> BTHome.add_measurement(:motion, true)\n\n    # With custom options\n    packet = BTHome.new_packet()\n    |> BTHome.add_measurement(:temperature, 74.21, unit: \"°F\")","ref":"BTHome.html#add_measurement/4-examples"},{"type":"function","title":"BTHome.deserialize/2","doc":"Deserializes BTHome v2 binary data into structured measurement data.\n\nParses BTHome v2 binary format and returns a `DecodedData` struct containing\nprotocol metadata and a list of `Measurement` structs. Handles error recovery\nfor unknown object IDs by skipping invalid data.\n\nFor encrypted data, provide decryption options to automatically decrypt measurements.","ref":"BTHome.html#deserialize/2"},{"type":"function","title":"Parameters - BTHome.deserialize/2","doc":"- `binary` - Binary data containing BTHome v2 encoded measurements\n- `opts` - Options for decryption (optional)\n  - `:key` - 16-byte encryption key\n  - `:mac_address` - 6-byte MAC address","ref":"BTHome.html#deserialize/2-parameters"},{"type":"function","title":"Returns - BTHome.deserialize/2","doc":"- `{:ok, decoded_data}` - Successfully parsed data with measurements\n- `{:error, error}` - Parsing failed with error details","ref":"BTHome.html#deserialize/2-returns"},{"type":"function","title":"Examples - BTHome.deserialize/2","doc":"# Basic deserialization\n    iex> {:ok, result} = BTHome.deserialize(<<64, 2, 41, 9, 3, 124, 26>>)\n    iex> result.version\n    2\n    iex> result.encrypted\n    false\n    iex> length(result.measurements)\n    2\n\n    # For encrypted data, provide key and MAC address\n    # opts = [key: < >, mac_address: < >]\n    # BTHome.deserialize(encrypted_binary, opts)\n\n    # With encryption flag\n    iex> BTHome.deserialize(<<65, 2, 41, 9>>)\n    {:ok, %BTHome.DecodedData{\n      version: 2,\n      encrypted: true,\n      trigger_based: false,\n      measurements: [],\n      ciphertext: <<2, 41, 9>>\n    }}\n\n    # Invalid data\n    iex> BTHome.deserialize(<<1, 2, 3>>)\n    {:error, \"Unsupported BTHome version: 0\"}","ref":"BTHome.html#deserialize/2-examples"},{"type":"function","title":"BTHome.deserialize_encrypted/3","doc":"Deserializes encrypted BTHome v2 binary data.\n\nThis is a convenience function for decrypting BTHome v2 data with the specified\ndecryption parameters.","ref":"BTHome.html#deserialize_encrypted/3"},{"type":"function","title":"Parameters - BTHome.deserialize_encrypted/3","doc":"- `binary` - Encrypted BTHome v2 binary data\n- `key` - 16-byte encryption key\n- `mac_address` - 6-byte MAC address","ref":"BTHome.html#deserialize_encrypted/3-parameters"},{"type":"function","title":"Returns - BTHome.deserialize_encrypted/3","doc":"- `{:ok, decoded_data}` - Successfully decrypted data with measurements\n- `{:error, error}` - Decryption failed with error details","ref":"BTHome.html#deserialize_encrypted/3-returns"},{"type":"function","title":"Examples - BTHome.deserialize_encrypted/3","doc":"key = encryption_key\n    mac = device_mac_address\n\n    {:ok, decoded} = BTHome.deserialize_encrypted(encrypted_binary, key, mac)","ref":"BTHome.html#deserialize_encrypted/3-examples"},{"type":"function","title":"BTHome.deserialize_measurements/2","doc":"Deserializes BTHome v2 binary data and returns measurements as a map.\n\nThis is a convenience function that extracts measurements from the decoded data\nand returns them as a map where keys are measurement types and values are the\nmeasured values. For measurements with multiple instances (same type, different\nobject IDs), the map will contain a list of values.","ref":"BTHome.html#deserialize_measurements/2"},{"type":"function","title":"Parameters - BTHome.deserialize_measurements/2","doc":"- `binary` - The BTHome v2 binary data to deserialize","ref":"BTHome.html#deserialize_measurements/2-parameters"},{"type":"function","title":"Returns - BTHome.deserialize_measurements/2","doc":"- `{:ok, map}` - A map of measurement types to values\n- `{:error, Error.t()}` - If deserialization fails","ref":"BTHome.html#deserialize_measurements/2-returns"},{"type":"function","title":"Examples - BTHome.deserialize_measurements/2","doc":"# Binary sensor measurement\n    iex> binary = <<0x40, 0x0F, 0x01>>\n    iex> BTHome.deserialize_measurements(binary)\n    {:ok, %{generic_boolean: true}}\n\n    # Temperature and humidity measurements\n    iex> binary = <<0x40, 0x02, 0xCA, 0x09, 0x03, 0xBF, 0x13>>\n    iex> {:ok, measurements} = BTHome.deserialize_measurements(binary)\n    iex> Map.has_key?(measurements, :temperature) and Map.has_key?(measurements, :humidity)\n    true\n\n    # Encrypted data (no measurements available)\n    iex> binary = <<0x41, 0x02, 0x29, 0x09>>\n    iex> BTHome.deserialize_measurements(binary)\n    {:ok, %{}}","ref":"BTHome.html#deserialize_measurements/2-examples"},{"type":"function","title":"BTHome.deserialize_measurements!/1","doc":"Deserializes BTHome v2 binary data and returns measurements as a map, raising on error.\n\nThis is the recommended function for most use cases as it provides the most ergonomic\nAPI for accessing measurement data. It returns measurements as a map where keys are\nmeasurement types and values are the measured values.","ref":"BTHome.html#deserialize_measurements!/1"},{"type":"function","title":"Parameters - BTHome.deserialize_measurements!/1","doc":"- `binary` - The BTHome v2 binary data to deserialize","ref":"BTHome.html#deserialize_measurements!/1-parameters"},{"type":"function","title":"Returns - BTHome.deserialize_measurements!/1","doc":"A map of measurement types to values. Raises `ArgumentError` on failure.","ref":"BTHome.html#deserialize_measurements!/1-returns"},{"type":"function","title":"Examples - BTHome.deserialize_measurements!/1","doc":"# Binary sensor measurement\n    iex> binary = <<0x40, 0x0F, 0x01>>\n    iex> BTHome.deserialize_measurements!(binary)\n    %{generic_boolean: true}\n\n    # Temperature and humidity measurements\n    iex> binary = <<0x40, 0x02, 0xCA, 0x09, 0x03, 0xBF, 0x13>>\n    iex> measurements = BTHome.deserialize_measurements!(binary)\n    iex> Map.has_key?(measurements, :temperature) and Map.has_key?(measurements, :humidity)\n    true\n\n    # Invalid data raises an error\n    iex> BTHome.deserialize_measurements!(<<1, 2, 3>>)\n    ** (ArgumentError) Unsupported BTHome version: 0","ref":"BTHome.html#deserialize_measurements!/1-examples"},{"type":"function","title":"BTHome.encryption_opts/1","doc":"Creates encryption options from a device context map.\n\nThis convenience function extracts encryption parameters from a device\ncontext map, making it easier to work with device management systems.","ref":"BTHome.html#encryption_opts/1"},{"type":"function","title":"Parameters - BTHome.encryption_opts/1","doc":"- `device_context` - Map containing device information with keys:\n  - `:key` or `:encryption_key` - 16-byte encryption key\n  - `:mac_address` or `:mac` - 6-byte MAC address\n  - `:counter` - 4-byte counter value","ref":"BTHome.html#encryption_opts/1-parameters"},{"type":"function","title":"Returns - BTHome.encryption_opts/1","doc":"- `{:ok, opts}` - Successfully created encryption options\n- `{:error, error}` - Missing or invalid parameters","ref":"BTHome.html#encryption_opts/1-returns"},{"type":"function","title":"Examples - BTHome.encryption_opts/1","doc":"device = %{\n      encryption_key: key,\n      mac_address: mac,\n      counter: 1\n    }\n\n    {:ok, opts} = BTHome.encryption_opts(device)\n    {:ok, binary} = BTHome.serialize(measurements, opts)","ref":"BTHome.html#encryption_opts/1-examples"},{"type":"function","title":"BTHome.encryption_opts/3","doc":"Creates encryption options for use with serialize/2.\n\nThis convenience function creates properly formatted encryption options\nfrom individual parameters, reducing verbosity and potential errors.","ref":"BTHome.html#encryption_opts/3"},{"type":"function","title":"Parameters - BTHome.encryption_opts/3","doc":"- `key` - 16-byte encryption key\n- `mac_address` - 6-byte MAC address\n- `counter` - 4-byte counter value","ref":"BTHome.html#encryption_opts/3-parameters"},{"type":"function","title":"Returns - BTHome.encryption_opts/3","doc":"Keyword list with encryption options ready for use with serialize/2.","ref":"BTHome.html#encryption_opts/3-returns"},{"type":"function","title":"Examples - BTHome.encryption_opts/3","doc":"key = BTHome.generate_encryption_key()\n    mac = <<0x12, 0x34, 0x56, 0x78, 0x9A, 0xBC>>\n\n    # Create encryption options\n    opts = BTHome.encryption_opts(key, mac, 1)\n\n    # Use with serialize/2\n    {:ok, binary} = BTHome.serialize(measurements, opts)","ref":"BTHome.html#encryption_opts/3-examples"},{"type":"function","title":"BTHome.generate_encryption_key/0","doc":"Generates a random encryption key for BTHome v2.","ref":"BTHome.html#generate_encryption_key/0"},{"type":"function","title":"Returns - BTHome.generate_encryption_key/0","doc":"16-byte random encryption key suitable for AES-128.","ref":"BTHome.html#generate_encryption_key/0-returns"},{"type":"function","title":"Examples - BTHome.generate_encryption_key/0","doc":"key = BTHome.generate_encryption_key()\n    byte_size(key)  # => 16","ref":"BTHome.html#generate_encryption_key/0-examples"},{"type":"function","title":"BTHome.key_from_hex/1","doc":"Converts a hex string to an encryption key.","ref":"BTHome.html#key_from_hex/1"},{"type":"function","title":"Parameters - BTHome.key_from_hex/1","doc":"- `hex_string` - 32-character hex string","ref":"BTHome.html#key_from_hex/1-parameters"},{"type":"function","title":"Returns - BTHome.key_from_hex/1","doc":"- `{:ok, key}` - Successfully converted key\n- `{:error, error}` - Invalid hex string","ref":"BTHome.html#key_from_hex/1-returns"},{"type":"function","title":"Examples - BTHome.key_from_hex/1","doc":"{:ok, key} = BTHome.key_from_hex(\"231d39c1d7cc1ab1aee224cd096db932\")","ref":"BTHome.html#key_from_hex/1-examples"},{"type":"function","title":"BTHome.key_to_hex/1","doc":"Converts an encryption key to a hex string.","ref":"BTHome.html#key_to_hex/1"},{"type":"function","title":"Parameters - BTHome.key_to_hex/1","doc":"- `key` - 16-byte encryption key","ref":"BTHome.html#key_to_hex/1-parameters"},{"type":"function","title":"Returns - BTHome.key_to_hex/1","doc":"Hex string representation of the key.","ref":"BTHome.html#key_to_hex/1-returns"},{"type":"function","title":"Examples - BTHome.key_to_hex/1","doc":"hex = BTHome.key_to_hex(key)","ref":"BTHome.html#key_to_hex/1-examples"},{"type":"function","title":"BTHome.measurement/3","doc":"Creates a validated measurement struct.\n\nThis is the recommended way to create measurements as it provides validation\nduring creation and automatically looks up metadata like units.","ref":"BTHome.html#measurement/3"},{"type":"function","title":"Parameters - BTHome.measurement/3","doc":"- `type` - The measurement type atom (must be supported)\n- `value` - The measurement value (number for sensors, boolean for binary sensors)\n- `opts` - Optional keyword list with `:unit` and `:object_id` overrides","ref":"BTHome.html#measurement/3-parameters"},{"type":"function","title":"Returns - BTHome.measurement/3","doc":"- `{:ok, measurement}` - Successfully created and validated measurement\n- `{:error, error}` - Creation failed with error details","ref":"BTHome.html#measurement/3-returns"},{"type":"function","title":"Examples - BTHome.measurement/3","doc":"# Environmental sensor\n    iex> BTHome.measurement(:temperature, 23.45)\n    {:ok, %BTHome.Measurement{type: :temperature, value: 23.45, unit: \"°C\"}}\n\n    # Binary sensor\n    iex> BTHome.measurement(:motion, true)\n    {:ok, %BTHome.Measurement{type: :motion, value: true, unit: nil}}\n\n    # With custom unit\n    iex> BTHome.measurement(:temperature, 74.21, unit: \"°F\")\n    {:ok, %BTHome.Measurement{type: :temperature, value: 74.21, unit: \"°F\"}}\n\n    # Invalid type\n    iex> BTHome.measurement(:invalid, 42)\n    {:error, \"Unsupported measurement type: :invalid\"}","ref":"BTHome.html#measurement/3-examples"},{"type":"function","title":"BTHome.new_encrypted_packet/3","doc":"Creates a new encrypted packet builder.\n\nThis convenience function creates a packet builder that can be used\nwith encryption. The encryption options are returned separately to be\nused with serialize/2.","ref":"BTHome.html#new_encrypted_packet/3"},{"type":"function","title":"Parameters - BTHome.new_encrypted_packet/3","doc":"- `key` - 16-byte encryption key\n- `mac_address` - 6-byte MAC address\n- `counter` - 4-byte counter value","ref":"BTHome.html#new_encrypted_packet/3-parameters"},{"type":"function","title":"Returns - BTHome.new_encrypted_packet/3","doc":"Tuple with packet struct and encryption options.","ref":"BTHome.html#new_encrypted_packet/3-returns"},{"type":"function","title":"Examples - BTHome.new_encrypted_packet/3","doc":"key = BTHome.generate_encryption_key()\n    mac = <<0x12, 0x34, 0x56, 0x78, 0x9A, 0xBC>>\n\n    {packet, opts} = BTHome.new_encrypted_packet(key, mac, 1)\n\n    {:ok, packet} = BTHome.add_measurement(packet, :temperature, 23.5)\n    {:ok, binary} = BTHome.serialize(packet, opts)","ref":"BTHome.html#new_encrypted_packet/3-examples"},{"type":"function","title":"BTHome.new_packet/0","doc":"Creates a new packet builder for the fluent/builder pattern API.\n\nThis function starts a new packet builder that can be used with the pipe\noperator to chain measurement additions and final serialization.","ref":"BTHome.html#new_packet/0"},{"type":"function","title":"Returns - BTHome.new_packet/0","doc":"A new packet builder struct.","ref":"BTHome.html#new_packet/0-returns"},{"type":"function","title":"Examples - BTHome.new_packet/0","doc":"# Basic builder pattern usage\n    {:ok, binary} = BTHome.new_packet()\n    |> BTHome.add_measurement(:temperature, 23.45)\n    |> BTHome.add_measurement(:motion, true)\n    |> BTHome.serialize()\n\n    # With encryption\n    {:ok, binary} = BTHome.new_packet()\n    |> BTHome.add_measurement(:temperature, 23.45)\n    |> BTHome.serialize(true)","ref":"BTHome.html#new_packet/0-examples"},{"type":"function","title":"BTHome.quick_serialize_encrypted/4","doc":"Quick serialization of measurements with encryption.\n\nThis convenience function provides a one-liner for serializing measurements\nwith encryption, reducing boilerplate code for simple use cases.","ref":"BTHome.html#quick_serialize_encrypted/4"},{"type":"function","title":"Parameters - BTHome.quick_serialize_encrypted/4","doc":"- `measurements` - List of measurements (structs or maps)\n- `key` - 16-byte encryption key\n- `mac_address` - 6-byte MAC address\n- `counter` - 4-byte counter value","ref":"BTHome.html#quick_serialize_encrypted/4-parameters"},{"type":"function","title":"Returns - BTHome.quick_serialize_encrypted/4","doc":"- `{:ok, binary}` - Successfully serialized encrypted binary\n- `{:error, error}` - Serialization failed with error details","ref":"BTHome.html#quick_serialize_encrypted/4-returns"},{"type":"function","title":"Examples - BTHome.quick_serialize_encrypted/4","doc":"{:ok, temp} = BTHome.measurement(:temperature, 23.5)\n    {:ok, humidity} = BTHome.measurement(:humidity, 65.0)\n    measurements = [temp, humidity]\n\n    {:ok, binary} = BTHome.quick_serialize_encrypted(\n      measurements, key, mac, 1\n    )","ref":"BTHome.html#quick_serialize_encrypted/4-examples"},{"type":"function","title":"BTHome.serialize/2","doc":"Serializes measurements into BTHome v2 binary format.","ref":"BTHome.html#serialize/2"},{"type":"function","title":"Parameters - BTHome.serialize/2","doc":"- `measurements_or_packet` - List of measurements or a `Packet` struct\n- `opts` - Serialization options (keyword list)\n  - `:encrypt` - Encryption options (keyword list or map, default: false)\n    - `:key` - 16-byte encryption key (required)\n    - `:mac_address` - 6-byte MAC address (required)\n    - `:counter` - 4-byte counter value (required)","ref":"BTHome.html#serialize/2-parameters"},{"type":"function","title":"Returns - BTHome.serialize/2","doc":"`{:ok, binary}` on success, `{:error, reason}` on failure.","ref":"BTHome.html#serialize/2-returns"},{"type":"function","title":"Examples - BTHome.serialize/2","doc":"# Basic usage\n    iex> {:ok, temp} = BTHome.measurement(:temperature, 23.45)\n    iex> {:ok, binary} = BTHome.serialize([temp])\n    iex> is_binary(binary)\n    true\n\n    # With encryption (keyword list - recommended)\n    iex> {:ok, temp} = BTHome.measurement(:temperature, 23.45)\n    iex> key = <<1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16>>\n    iex> mac = <<0xAA, 0xBB, 0xCC, 0xDD, 0xEE, 0xFF>>\n    iex> opts = [encrypt: [key: key, mac_address: mac, counter: 1]]\n    iex> {:ok, binary} = BTHome.serialize([temp], opts)\n    iex> < > = binary\n    iex> Bitwise.band(device_info, 0x01) == 1\n    true\n\n    # With encryption (map format)\n    iex> {:ok, temp} = BTHome.measurement(:temperature, 23.45)\n    iex> key = <<1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16>>\n    iex> mac = <<0xAA, 0xBB, 0xCC, 0xDD, 0xEE, 0xFF>>\n    iex> opts = [encrypt: %{key: key, mac_address: mac, counter: 1}]\n    iex> {:ok, binary} = BTHome.serialize([temp], opts)\n    iex> is_binary(binary)\n    true","ref":"BTHome.html#serialize/2-examples"},{"type":"function","title":"BTHome.serialize!/2","doc":"Serializes measurements or packet, raising on error.\n\nSimilar to `serialize/2` but raises an exception instead of returning an error tuple.\nThis is useful for pipeline operations where you want to fail fast on errors.","ref":"BTHome.html#serialize!/2"},{"type":"function","title":"Parameters - BTHome.serialize!/2","doc":"- `measurements_or_packet` - List of measurements or a Packet struct\n- `opts` - Serialization options (same as `serialize/2`)","ref":"BTHome.html#serialize!/2-parameters"},{"type":"function","title":"Returns - BTHome.serialize!/2","doc":"Binary BTHome v2 data on success, raises on error.","ref":"BTHome.html#serialize!/2-returns"},{"type":"function","title":"Examples - BTHome.serialize!/2","doc":"# Basic usage\n    iex> {:ok, temp} = BTHome.measurement(:temperature, 23.45)\n    iex> binary = BTHome.serialize!([temp])\n    iex> is_binary(binary)\n    true\n\n    # With builder pattern\n    iex> binary = BTHome.new_packet()\n    iex> |> BTHome.add_measurement(:temperature, 23.45)\n    iex> |> BTHome.serialize!()\n    iex> is_binary(binary)\n    true\n\n    # With encryption\n    iex> {:ok, temp} = BTHome.measurement(:temperature, 23.45)\n    iex> key = <<1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16>>\n    iex> mac = <<0xAA, 0xBB, 0xCC, 0xDD, 0xEE, 0xFF>>\n    iex> opts = [encrypt: [key: key, mac_address: mac, counter: 1]]\n    iex> binary = BTHome.serialize!([temp], opts)\n    iex> is_binary(binary)\n    true","ref":"BTHome.html#serialize!/2-examples"},{"type":"function","title":"Raises - BTHome.serialize!/2","doc":"`ArgumentError` if serialization fails.","ref":"BTHome.html#serialize!/2-raises"},{"type":"function","title":"BTHome.serialize_encrypted/2","doc":"Serializes an encrypted packet to binary format.\n\nThis convenience function serializes a packet with the provided\nencryption options, providing a cleaner API for encrypted packets.","ref":"BTHome.html#serialize_encrypted/2"},{"type":"function","title":"Parameters - BTHome.serialize_encrypted/2","doc":"- `packet` - Packet struct\n- `encryption_opts` - Encryption options from encryption_opts/3","ref":"BTHome.html#serialize_encrypted/2-parameters"},{"type":"function","title":"Returns - BTHome.serialize_encrypted/2","doc":"- `{:ok, binary}` - Successfully serialized encrypted binary\n- `{:error, error}` - Serialization failed with error details","ref":"BTHome.html#serialize_encrypted/2-returns"},{"type":"function","title":"Examples - BTHome.serialize_encrypted/2","doc":"{packet, opts} = BTHome.new_encrypted_packet(key, mac, 1)\n    {:ok, packet} = BTHome.add_measurement(packet, :temperature, 23.5)\n\n    {:ok, binary} = BTHome.serialize_encrypted(packet, opts)","ref":"BTHome.html#serialize_encrypted/2-examples"},{"type":"function","title":"BTHome.serialize_encrypted/4","doc":"Serializes measurements with encryption.\n\nThis is a convenience function for encrypting BTHome v2 data with the specified\nencryption parameters.","ref":"BTHome.html#serialize_encrypted/4"},{"type":"function","title":"Parameters - BTHome.serialize_encrypted/4","doc":"- `measurements` - List of measurements to encrypt\n- `key` - 16-byte encryption key\n- `mac_address` - 6-byte MAC address\n- `counter` - 4-byte counter value","ref":"BTHome.html#serialize_encrypted/4-parameters"},{"type":"function","title":"Returns - BTHome.serialize_encrypted/4","doc":"- `{:ok, binary}` - Successfully encrypted BTHome v2 binary data\n- `{:error, error}` - Encryption failed with error details","ref":"BTHome.html#serialize_encrypted/4-returns"},{"type":"function","title":"Examples - BTHome.serialize_encrypted/4","doc":"key = :crypto.strong_rand_bytes(16)\n    mac = <<0x54, 0x48, 0xe6, 0x8f, 0x80, 0xa5>>\n    {:ok, temp} = BTHome.measurement(:temperature, 23.45)\n\n    {:ok, encrypted} = BTHome.serialize_encrypted([temp], key, mac, 1)","ref":"BTHome.html#serialize_encrypted/4-examples"},{"type":"function","title":"BTHome.serialize_for_device/2","doc":"Serializes measurements for a specific device.\n\nThis convenience function integrates with device management by accepting\na device context map containing encryption parameters.","ref":"BTHome.html#serialize_for_device/2"},{"type":"function","title":"Parameters - BTHome.serialize_for_device/2","doc":"- `measurements` - List of measurements (structs or maps)\n- `device_context` - Map containing device information","ref":"BTHome.html#serialize_for_device/2-parameters"},{"type":"function","title":"Returns - BTHome.serialize_for_device/2","doc":"- `{:ok, binary}` - Successfully serialized binary (encrypted if device has encryption)\n- `{:error, error}` - Serialization failed with error details","ref":"BTHome.html#serialize_for_device/2-returns"},{"type":"function","title":"Examples - BTHome.serialize_for_device/2","doc":"device = %{\n      encryption_key: key,\n      mac_address: mac,\n      counter: 1\n    }\n\n    {:ok, temp} = BTHome.measurement(:temperature, 23.5)\n    {:ok, binary} = BTHome.serialize_for_device([temp], device)","ref":"BTHome.html#serialize_for_device/2-examples"},{"type":"function","title":"BTHome.supported_types/0","doc":"Returns all supported measurement types and their properties.\n\nProvides a map of measurement types to their complete definitions including\nunits, scaling factors, data sizes, and other metadata.","ref":"BTHome.html#supported_types/0"},{"type":"function","title":"Returns - BTHome.supported_types/0","doc":"A map where keys are measurement type atoms and values are definition maps.","ref":"BTHome.html#supported_types/0-returns"},{"type":"function","title":"Examples - BTHome.supported_types/0","doc":"iex> types = BTHome.supported_types()\n    iex> Map.has_key?(types, :temperature)\n    true\n    iex> types[:temperature].unit\n    \"°C\"","ref":"BTHome.html#supported_types/0-examples"},{"type":"function","title":"BTHome.validate_measurement/1","doc":"Validates a single measurement for serialization.\n\nChecks that the measurement type is supported and the value is valid\nfor that type (correct data type and within acceptable range).","ref":"BTHome.html#validate_measurement/1"},{"type":"function","title":"Parameters - BTHome.validate_measurement/1","doc":"- `measurement` - A measurement struct or map to validate","ref":"BTHome.html#validate_measurement/1-parameters"},{"type":"function","title":"Returns - BTHome.validate_measurement/1","doc":"- `:ok` - Measurement is valid\n- `{:error, error}` - Validation failed with error details","ref":"BTHome.html#validate_measurement/1-returns"},{"type":"function","title":"Examples - BTHome.validate_measurement/1","doc":"iex> {:ok, temp} = BTHome.measurement(:temperature, 23.45)\n    iex> BTHome.validate_measurement(temp)\n    :ok\n\n    iex> invalid = %BTHome.Measurement{type: :invalid, value: 42}\n    iex> BTHome.validate_measurement(invalid)\n    {:error, \"Unsupported measurement type: :invalid\"}","ref":"BTHome.html#validate_measurement/1-examples"},{"type":"function","title":"BTHome.validate_measurements/1","doc":"Validates a list of measurements for serialization.\n\nValidates each measurement in the list and ensures the overall list\nis suitable for serialization.","ref":"BTHome.html#validate_measurements/1"},{"type":"function","title":"Parameters - BTHome.validate_measurements/1","doc":"- `measurements` - List of measurement structs or maps to validate","ref":"BTHome.html#validate_measurements/1-parameters"},{"type":"function","title":"Returns - BTHome.validate_measurements/1","doc":"- `:ok` - All measurements are valid\n- `{:error, error}` - Validation failed with error details","ref":"BTHome.html#validate_measurements/1-returns"},{"type":"function","title":"Examples - BTHome.validate_measurements/1","doc":"iex> {:ok, temp} = BTHome.measurement(:temperature, 23.45)\n    iex> {:ok, humidity} = BTHome.measurement(:humidity, 67.8)\n    iex> BTHome.validate_measurements([temp, humidity])\n    :ok\n\n    iex> invalid = %BTHome.Measurement{type: :invalid, value: 42}\n    iex> BTHome.validate_measurements([invalid])\n    {:error, \"Measurement 0: Unsupported measurement type: :invalid\"}","ref":"BTHome.html#validate_measurements/1-examples"},{"type":"module","title":"BTHome.DecodedData","doc":"Struct representing decoded BTHome v2 data with type safety.\n\nThis struct contains the complete result of deserializing a BTHome v2 binary payload.\nIt includes protocol metadata and all decoded measurements.","ref":"BTHome.DecodedData.html"},{"type":"module","title":"Fields - BTHome.DecodedData","doc":"- `:version` - BTHome protocol version (should be 2)\n- `:encrypted` - Whether the payload was encrypted\n- `:trigger_based` - Whether this is a trigger-based transmission\n- `:measurements` - List of decoded measurements as `BTHome.Measurement` structs\n- `:ciphertext` - Raw encrypted data (only present for encrypted payloads)\n- `:mac_reversed` - MAC address in reversed byte order (when available)\n- `:counter` - Encryption counter (only present for encrypted payloads)\n- `:mic` - Message integrity check (only present for encrypted payloads)","ref":"BTHome.DecodedData.html#module-fields"},{"type":"module","title":"Examples - BTHome.DecodedData","doc":"# Typical decoded data\n    %BTHome.DecodedData{\n      version: 2,\n      encrypted: false,\n      trigger_based: false,\n      measurements: [\n        %BTHome.Measurement{type: :temperature, value: 23.45, unit: \"°C\"},\n        %BTHome.Measurement{type: :humidity, value: 67.8, unit: \"%\"}\n      ]\n    }","ref":"BTHome.DecodedData.html#module-examples"},{"type":"type","title":"BTHome.DecodedData.t/0","doc":"","ref":"BTHome.DecodedData.html#t:t/0"},{"type":"module","title":"BTHome.Measurement","doc":"Struct representing a BTHome v2 measurement with type safety.\n\nThis struct provides a type-safe representation of sensor measurements in the BTHome v2\nprotocol. It enforces required fields and provides validation during creation.","ref":"BTHome.Measurement.html"},{"type":"module","title":"Fields - BTHome.Measurement","doc":"- `:type` (required) - The sensor type as an atom (e.g., `:temperature`, `:humidity`)\n- `:value` (required) - The measured value (number for sensors, boolean for binary sensors)\n- `:unit` (optional) - The unit of measurement as a string (e.g., \"°C\", \"%\")\n- `:object_id` (optional) - The BTHome object ID for this measurement type","ref":"BTHome.Measurement.html#module-fields"},{"type":"module","title":"Examples - BTHome.Measurement","doc":"# Create a temperature measurement\n    iex> BTHome.Measurement.new(:temperature, 23.45)\n    {:ok, %BTHome.Measurement{type: :temperature, value: 23.45, unit: \"°C\"}}\n\n    # Create a binary sensor measurement\n    iex> BTHome.Measurement.new(:motion, true)\n    {:ok, %BTHome.Measurement{type: :motion, value: true, unit: nil}}\n\n    # Invalid measurement type\n    iex> BTHome.Measurement.new(:invalid_type, 42)\n    {:error, \"Unsupported measurement type: invalid_type\"}","ref":"BTHome.Measurement.html#module-examples"},{"type":"function","title":"BTHome.Measurement.new/3","doc":"Creates a new measurement with validation.\n\nThis function creates a new `BTHome.Measurement` struct and validates it according\nto BTHome v2 protocol requirements. It automatically looks up the unit for known\nmeasurement types.","ref":"BTHome.Measurement.html#new/3"},{"type":"function","title":"Parameters - BTHome.Measurement.new/3","doc":"- `type` - The measurement type as an atom (must be supported by BTHome v2)\n- `value` - The measurement value (number for sensors, boolean for binary sensors)\n- `opts` - Optional keyword list with:\n  - `:unit` - Override the default unit for this measurement type\n  - `:object_id` - Override the default object ID for this measurement type","ref":"BTHome.Measurement.html#new/3-parameters"},{"type":"function","title":"Returns - BTHome.Measurement.new/3","doc":"- `{:ok, measurement}` - Successfully created and validated measurement\n- `{:error, error}` - Validation failed with error details","ref":"BTHome.Measurement.html#new/3-returns"},{"type":"function","title":"Examples - BTHome.Measurement.new/3","doc":"# Temperature measurement (unit automatically set to \"°C\")\n    iex> BTHome.Measurement.new(:temperature, 23.45)\n    {:ok, %BTHome.Measurement{type: :temperature, value: 23.45, unit: \"°C\"}}\n\n    # Binary sensor\n    iex> BTHome.Measurement.new(:motion, true)\n    {:ok, %BTHome.Measurement{type: :motion, value: true, unit: nil}}\n\n    # With custom unit\n    iex> BTHome.Measurement.new(:temperature, 74.21, unit: \"°F\")\n    {:ok, %BTHome.Measurement{type: :temperature, value: 74.21, unit: \"°F\"}}\n\n    # Invalid type\n    iex> BTHome.Measurement.new(:invalid, 42)\n    {:error, \"Unsupported measurement type: :invalid_type\"}","ref":"BTHome.Measurement.html#new/3-examples"},{"type":"type","title":"BTHome.Measurement.t/0","doc":"","ref":"BTHome.Measurement.html#t:t/0"},{"type":"module","title":"BTHome.Config","doc":"Configuration constants for BTHome v2 protocol.\n\nThis module centralizes all magic numbers, protocol-specific values, and configuration\nconstants used throughout the BTHome v2 implementation. By keeping these values in one\nplace, we ensure consistency and make the codebase easier to maintain.","ref":"BTHome.Config.html"},{"type":"module","title":"Constants - BTHome.Config","doc":"- `version/0` - BTHome protocol version (2)\n- `version_mask/0` - Bitmask for extracting version from device info (0xE0)\n- `version_shift/0` - Bit shift for version extraction (5)\n- `trigger_mask/0` - Bitmask for trigger-based flag (0x10)\n- `encryption_mask/0` - Bitmask for encryption flag (0x01)\n- `max_skip_bytes/0` - Maximum bytes to skip when recovering from unknown data (10)\n- `device_info_base/0` - Base value for device info byte (0x40)","ref":"BTHome.Config.html#module-constants"},{"type":"module","title":"Examples - BTHome.Config","doc":"iex> BTHome.Config.version()\n    2\n\n    iex> BTHome.Config.version_mask()\n    224  # 0xE0","ref":"BTHome.Config.html#module-examples"},{"type":"function","title":"BTHome.Config.device_info_base/0","doc":"","ref":"BTHome.Config.html#device_info_base/0"},{"type":"function","title":"BTHome.Config.encryption_mask/0","doc":"","ref":"BTHome.Config.html#encryption_mask/0"},{"type":"function","title":"BTHome.Config.max_skip_bytes/0","doc":"","ref":"BTHome.Config.html#max_skip_bytes/0"},{"type":"function","title":"BTHome.Config.trigger_mask/0","doc":"","ref":"BTHome.Config.html#trigger_mask/0"},{"type":"function","title":"BTHome.Config.version/0","doc":"","ref":"BTHome.Config.html#version/0"},{"type":"function","title":"BTHome.Config.version_mask/0","doc":"","ref":"BTHome.Config.html#version_mask/0"},{"type":"function","title":"BTHome.Config.version_shift/0","doc":"","ref":"BTHome.Config.html#version_shift/0"},{"type":"module","title":"BTHome.Decoder","doc":"Decoder for BTHome v2 format.\n\nThis module provides functions to decode BTHome v2 binary data into structured\nmeasurement data.","ref":"BTHome.Decoder.html"},{"type":"function","title":"BTHome.Decoder.decode_measurements/2","doc":"Decode BTHome v2 binary data","ref":"BTHome.Decoder.html#decode_measurements/2"},{"type":"function","title":"BTHome.Decoder.decode_with_decryption/3","doc":"Decode BTHome v2 binary data with decryption support.","ref":"BTHome.Decoder.html#decode_with_decryption/3"},{"type":"function","title":"Options - BTHome.Decoder.decode_with_decryption/3","doc":"- `:key` - 16-byte encryption key (required for encrypted data)\n- `:mac_address` - 6-byte MAC address (required for encrypted data)","ref":"BTHome.Decoder.html#decode_with_decryption/3-options"},{"type":"function","title":"Examples - BTHome.Decoder.decode_with_decryption/3","doc":"# Unencrypted data\n    {:ok, decoded} = decode_measurements(data)\n\n    # Encrypted data\n    opts = [key: encryption_key, mac_address: device_mac]\n    {:ok, decoded} = decode_measurements(encrypted_data, opts)","ref":"BTHome.Decoder.html#decode_with_decryption/3-examples"},{"type":"module","title":"BTHome.Encoder","doc":"BTHome v2 serialization logic.\nHandles encoding measurements into binary format with struct support.","ref":"BTHome.Encoder.html"},{"type":"function","title":"BTHome.Encoder.encode_measurements/2","doc":"Encode measurements to BTHome v2 binary format","ref":"BTHome.Encoder.html#encode_measurements/2"},{"type":"function","title":"BTHome.Encoder.encode_validated/2","doc":"Encode pre-validated measurements (skips validation)","ref":"BTHome.Encoder.html#encode_validated/2"},{"type":"module","title":"BTHome.Objects","doc":"BTHome v2 object definitions and metadata.\n\nThis module centralizes all sensor type definitions according to the BTHome v2 specification\nand provides fast lookups. It includes metadata for each sensor type including units,\nscaling factors, data sizes, and signedness.","ref":"BTHome.Objects.html"},{"type":"module","title":"Sensor Categories - BTHome.Objects","doc":"","ref":"BTHome.Objects.html#module-sensor-categories"},{"type":"module","title":"Environmental Sensors (0x01-0x0F) - BTHome.Objects","doc":"- Battery level, temperature, humidity, pressure, illuminance\n- Mass measurements, dewpoint, energy, power, voltage\n- Air quality sensors (PM2.5, PM10)","ref":"BTHome.Objects.html#module-environmental-sensors-0x01-0x0f"},{"type":"module","title":"Binary Sensors (0x10-0x2D) - BTHome.Objects","doc":"- Motion, door/window, occupancy, presence detection\n- Safety sensors (smoke, CO, gas, tamper)\n- Device state sensors (battery low, charging, connectivity)","ref":"BTHome.Objects.html#module-binary-sensors-0x10-0x2d"},{"type":"module","title":"Performance Optimizations - BTHome.Objects","doc":"This module uses compile-time optimizations to provide O(1) lookups:\n- `@type_to_id_map` - Fast type-to-ID resolution\n- `@binary_sensor_types` - Set-based binary sensor detection\n- `@supported_types_map` - Efficient type validation","ref":"BTHome.Objects.html#module-performance-optimizations"},{"type":"module","title":"Examples - BTHome.Objects","doc":"# Get definition by object ID\n    iex> BTHome.Objects.get_definition(0x02)\n    %{name: :temperature, unit: \"°C\", factor: 0.01, signed: true, size: 2}\n\n    # Find by measurement type\n    iex> BTHome.Objects.find_by_type(:temperature)\n    {2, %{name: :temperature, unit: \"°C\", factor: 0.01, signed: true, size: 2}}\n\n    # Check if binary sensor\n    iex> BTHome.Objects.binary_sensor?(:motion)\n    true","ref":"BTHome.Objects.html#module-examples"},{"type":"function","title":"BTHome.Objects.binary_sensor?/1","doc":"Checks if a measurement type is a binary sensor (O(1) lookup).\n\nBinary sensors have empty units and 1-byte size, representing boolean states.","ref":"BTHome.Objects.html#binary_sensor?/1"},{"type":"function","title":"Examples - BTHome.Objects.binary_sensor?/1","doc":"iex> BTHome.Objects.binary_sensor?(:motion)\n    true\n\n    iex> BTHome.Objects.binary_sensor?(:temperature)\n    false","ref":"BTHome.Objects.html#binary_sensor?/1-examples"},{"type":"function","title":"BTHome.Objects.find_by_type/1","doc":"Finds object ID and definition by measurement type (O(1) lookup).\n\nReturns a tuple of {object_id, definition} or nil if not found.","ref":"BTHome.Objects.html#find_by_type/1"},{"type":"function","title":"Examples - BTHome.Objects.find_by_type/1","doc":"iex> BTHome.Objects.find_by_type(:temperature)\n    {2, %{name: :temperature, unit: \"°C\", factor: 0.01, signed: true, size: 2}}\n\n    iex> BTHome.Objects.find_by_type(:invalid)\n    nil","ref":"BTHome.Objects.html#find_by_type/1-examples"},{"type":"function","title":"BTHome.Objects.get_all_definitions/0","doc":"Returns all object definitions as a map.","ref":"BTHome.Objects.html#get_all_definitions/0"},{"type":"function","title":"Examples - BTHome.Objects.get_all_definitions/0","doc":"iex> definitions = BTHome.Objects.get_all_definitions()\n    iex> Map.has_key?(definitions, 0x02)\n    true","ref":"BTHome.Objects.html#get_all_definitions/0-examples"},{"type":"function","title":"BTHome.Objects.get_definition/1","doc":"Gets object definition by BTHome object ID.\n\nReturns the complete definition including name, unit, factor, signedness, and size.","ref":"BTHome.Objects.html#get_definition/1"},{"type":"function","title":"Examples - BTHome.Objects.get_definition/1","doc":"iex> BTHome.Objects.get_definition(0x02)\n    %{name: :temperature, unit: \"°C\", factor: 0.01, signed: true, size: 2}\n\n    iex> BTHome.Objects.get_definition(0xFF)\n    nil","ref":"BTHome.Objects.html#get_definition/1-examples"},{"type":"function","title":"BTHome.Objects.get_supported_types/0","doc":"Returns all supported measurement types.","ref":"BTHome.Objects.html#get_supported_types/0"},{"type":"function","title":"Examples - BTHome.Objects.get_supported_types/0","doc":"iex> types = BTHome.Objects.get_supported_types()\n    iex> :temperature in types\n    true","ref":"BTHome.Objects.html#get_supported_types/0-examples"},{"type":"module","title":"BTHome.Validator","doc":"Input validation for BTHome v2 measurements and data.\nProvides comprehensive validation with detailed error messages and struct support.","ref":"BTHome.Validator.html"},{"type":"function","title":"BTHome.Validator.validate_device_info/1","doc":"Validate BTHome device information byte","ref":"BTHome.Validator.html#validate_device_info/1"},{"type":"function","title":"BTHome.Validator.validate_measurement/1","doc":"Validate a single measurement (struct-only)","ref":"BTHome.Validator.html#validate_measurement/1"},{"type":"function","title":"BTHome.Validator.validate_measurements/1","doc":"Validate a list of measurements","ref":"BTHome.Validator.html#validate_measurements/1"},{"type":"extras","title":"BTHome v2 Parser/Serializer for Elixir","doc":"# BTHome v2 Parser/Serializer for Elixir\n\nA comprehensive, type-safe implementation of the BTHome v2 protocol for Elixir. This library provides serialization and deserialization of sensor data according to the [BTHome v2 specification](https://bthome.io/).","ref":"readme.html"},{"type":"extras","title":"Features - BTHome v2 Parser/Serializer for Elixir","doc":"- 🔒 **Type Safety** - Uses structs for measurements and decoded data\n- ✅ **Validation** - Comprehensive validation of measurement types and values\n- 🚨 **Error Handling** - Structured errors with context information\n- ⚡ **Performance** - Compile-time optimizations for fast lookups\n- 🔄 **Compatibility** - Supports both struct and map-based APIs\n- 📊 **Complete Coverage** - Supports all BTHome v2 sensor types\n- 🛡️ **Error Recovery** - Graceful handling of unknown object IDs","ref":"readme.html#features"},{"type":"extras","title":"Installation - BTHome v2 Parser/Serializer for Elixir","doc":"Add `bthome` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:bthome, \"~> 0.1.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Quick Start - BTHome v2 Parser/Serializer for Elixir","doc":"```elixir\n# Create measurements using the builder pattern (recommended)\nbinary = BTHome.new_packet()\n|> BTHome.add_measurement(:temperature, 23.45)\n|> BTHome.add_measurement(:motion, true)\n|> BTHome.serialize!()\n# => <<64, 2, 41, 9, 33, 1>>\n\n# Serialize to binary format\n{:ok, binary} = BTHome.serialize([temp, motion])\n# => {:ok, <<64, 2, 41, 9, 33, 1>>}\n\n# Deserialize back to structs\n{:ok, decoded} = BTHome.deserialize(binary)\n# => {:ok, %BTHome.DecodedData{\n#      version: 2,\n#      encrypted: false,\n#      trigger_based: false,\n#      measurements: [\n#        %BTHome.Measurement{type: :temperature, value: 23.45, unit: \"°C\"},\n#        %BTHome.Measurement{type: :motion, value: true, unit: nil}\n#      ]\n#    }}\n# Deserialize back (recommended)\nmeasurements = BTHome.deserialize_measurements!(binary)\n# => %{temperature: 23.45, motion: true}\n\n# Direct access to values\ntemp = measurements.temperature  # 23.45\nmotion = measurements.motion     # true\n```","ref":"readme.html#quick-start"},{"type":"extras","title":"Supported Sensor Types - BTHome v2 Parser/Serializer for Elixir","doc":"","ref":"readme.html#supported-sensor-types"},{"type":"extras","title":"Environmental Sensors - BTHome v2 Parser/Serializer for Elixir","doc":"| Type | Unit | Description | Range |\n|------|------|-------------|-------|\n| `:temperature` | °C | Temperature | -327.68 to 327.67 °C |\n| `:humidity` | % | Relative humidity | 0 to 655.35% |\n| `:pressure` | hPa | Atmospheric pressure | 0 to 167772.15 hPa |\n| `:illuminance` | lux | Light level | 0 to 167772.15 lux |\n| `:battery` | % | Battery level | 0 to 255% |\n| `:energy` | kWh | Energy consumption | 0 to 16777.215 kWh |\n| `:power` | W | Power consumption | 0 to 167772.15 W |\n| `:voltage` | V | Voltage | 0 to 65.535 V |\n| `:pm2_5` | µg/m³ | PM2.5 particles | 0 to 65535 µg/m³ |\n| `:pm10` | µg/m³ | PM10 particles | 0 to 65535 µg/m³ |\n| `:co2` | ppm | CO2 concentration | 0 to 65535 ppm |\n| `:tvoc` | µg/m³ | Total VOC | 0 to 65535 µg/m³ |","ref":"readme.html#environmental-sensors"},{"type":"extras","title":"Binary Sensors - BTHome v2 Parser/Serializer for Elixir","doc":"| Type | Description |\n|------|-------------|\n| `:motion` | Motion detection |\n| `:door` | Door state (open/closed) |\n| `:window` | Window state (open/closed) |\n| `:occupancy` | Room occupancy |\n| `:presence` | Presence detection |\n| `:smoke` | Smoke detection |\n| `:gas_detected` | Gas detection |\n| `:carbon_monoxide` | CO detection |\n| `:battery_low` | Low battery warning |\n| `:battery_charging` | Charging status |\n| `:connectivity` | Connection status |\n| `:problem` | Problem/fault status |\n| `:safety` | Safety status |\n| `:tamper` | Tamper detection |\n| `:vibration` | Vibration detection |","ref":"readme.html#binary-sensors"},{"type":"extras","title":"API Documentation - BTHome v2 Parser/Serializer for Elixir","doc":"","ref":"readme.html#api-documentation"},{"type":"extras","title":"Creating Measurements - BTHome v2 Parser/Serializer for Elixir","doc":"```elixir\n# Recommended: Using the measurement function with validation\n{:ok, temp} = BTHome.measurement(:temperature, 23.45)\n{:ok, motion} = BTHome.measurement(:motion, true)\n\n# With custom unit override\n{:ok, temp_f} = BTHome.measurement(:temperature, 74.21, unit: \"°F\")\n\n# Direct struct creation (advanced)\n%BTHome.Measurement{\n  type: :temperature,\n  value: 23.45,\n  unit: \"°C\"\n}\n```","ref":"readme.html#creating-measurements"},{"type":"extras","title":"Serialization - BTHome v2 Parser/Serializer for Elixir","doc":"```elixir\n# Basic serialization\nmeasurements = [temp, motion]\n{:ok, binary} = BTHome.serialize(measurements)\n\n# With encryption flag\n{:ok, encrypted_binary} = BTHome.serialize(measurements, true)\n\n# Legacy map format (still supported)\nlegacy_measurements = [\n  %{type: :temperature, value: 23.45},\n  %{type: :humidity, value: 67.8}\n]\n{:ok, binary} = BTHome.serialize(legacy_measurements)\n```","ref":"readme.html#serialization"},{"type":"extras","title":"Builder Pattern API - BTHome v2 Parser/Serializer for Elixir","doc":"For a more fluent, pipeable approach to creating BTHome packets:\n\n```elixir\n# Create and serialize in a single pipeline\n{:ok, binary} = BTHome.new_packet()\n|> BTHome.add_measurement(:temperature, 23.45)\n|> BTHome.add_measurement(:motion, true)\n|> BTHome.add_measurement(:humidity, 67.8)\n|> BTHome.serialize()\n\n# With encryption\n{:ok, encrypted_binary} = BTHome.new_packet()\n|> BTHome.add_measurement(:temperature, 23.45)\n|> BTHome.add_measurement(:battery, 85)\n|> BTHome.serialize(true)\n\n# Error handling with builder pattern\nresult = BTHome.new_packet()\n|> BTHome.add_measurement(:temperature, 23.45)\n|> BTHome.add_measurement(:invalid_type, 42)  # This will cause an error\n|> BTHome.serialize()\n\ncase result do\n  {:ok, binary} -> process_binary(binary)\n  {:error, error} -> handle_error(error)\nend\n\n# Custom measurement options\n{:ok, binary} = BTHome.new_packet()\n|> BTHome.add_measurement(:temperature, 74.21, unit: \"°F\")\n|> BTHome.add_measurement(:voltage, 3.3, object_id: 0x0C)\n|> BTHome.serialize()\n```\n\nThe builder pattern provides several advantages:\n- **Fluent API**: Chain operations naturally with the pipe operator\n- **Error Accumulation**: Invalid measurements are stored but don't break the chain\n- **Validation**: Each measurement is validated when added\n- **Flexibility**: Mix with existing APIs or use standalone","ref":"readme.html#builder-pattern-api"},{"type":"extras","title":"Deserialization - BTHome v2 Parser/Serializer for Elixir","doc":"**Recommended: Use `deserialize_measurements!/1` for the most ergonomic API:**\n\n```elixir\n# Simple, direct access to measurements\nbinary = <<64, 2, 202, 9, 3, 191, 19>>\nmeasurements = BTHome.deserialize_measurements!(binary)\n\n# Direct access to values\ntemp = measurements.temperature  # 25.06\nhumidity = measurements.humidity # 50.23\n\n# Handle multiple measurements of the same type\nmeasurements.voltage  # [3.1, 2.8] - list for multiple values\n\n# Binary sensors\nmeasurements.motion   # true/false\n```\n\n**Alternative: Use the tuple-returning version for explicit error handling:**\n\n```elixir\ncase BTHome.deserialize_measurements(binary) do\n  {:ok, measurements} -> \n    IO.puts(\"Temperature: #{measurements.temperature}°C\")\n  {:error, reason} -> \n    IO.puts(\"Failed to decode: #{reason}\")\nend\n```\n\n**Low-level: Access the full decoded structure:**\n\n```elixir\n{:ok, decoded} = BTHome.deserialize(binary)\n\n# Access metadata\ndecoded.version        # => 2\ndecoded.encrypted      # => false\ndecoded.trigger_based  # => false\n\n# Access measurements as structs\n[temp_measurement, humidity_measurement] = decoded.measurements\ntemp_measurement.value  # 25.06\ntemp_measurement.unit   # \"°C\"\n```","ref":"readme.html#deserialization"},{"type":"extras","title":"Convenience Functions - BTHome v2 Parser/Serializer for Elixir","doc":"#### `deserialize_measurements!/1` (Recommended)\n\nThe most ergonomic way to access measurement data. Returns measurements as a map and raises on error:\n\n```elixir\n# Simple, direct access - no pattern matching needed\nbinary = <<64, 2, 202, 9, 3, 191, 19>>\nmeasurements = BTHome.deserialize_measurements!(binary)\n\n# Direct access to values\ntemp = measurements.temperature  # 25.06\nhumidity = measurements.humidity # 50.23\n\n# Handle multiple measurements of the same type\nbinary_with_multiple_temps = <<64, 2, 202, 9, 18, 2, 100, 5>>\nmeasurements = BTHome.deserialize_measurements!(binary_with_multiple_temps)\nmeasurements.temperature  # [25.06, 13.0] - returns list for multiple values\n\n# Binary sensors\nbinary_sensor = <<64, 15, 1>>\nmeasurements = BTHome.deserialize_measurements!(binary_sensor)\nmeasurements.generic_boolean  # true\n```\n\n#### `deserialize_measurements/1`\n\nFor explicit error handling, use the tuple-returning version:\n\n```elixir\ncase BTHome.deserialize_measurements(binary) do\n  {:ok, measurements} -> \n    # Process measurements\n    IO.puts(\"Temperature: #{measurements.temperature}°C\")\n  {:error, %BTHome.Error{message: message}} -> \n    IO.puts(\"Decoding failed: #{message}\")\nend\n```","ref":"readme.html#convenience-functions"},{"type":"extras","title":"Validation - BTHome v2 Parser/Serializer for Elixir","doc":"```elixir\n# Validate individual measurement\ncase BTHome.validate_measurement(measurement) do\n  :ok -> IO.puts(\"Valid measurement\")\n  {:error, error} -> IO.puts(\"Invalid: #{error.message}\")\nend\n\n# Validate list of measurements\ncase BTHome.validate_measurements(measurements) do\n  :ok -> BTHome.serialize(measurements)\n  {:error, error} -> handle_validation_error(error)\nend\n```","ref":"readme.html#validation"},{"type":"extras","title":"Error Handling - BTHome v2 Parser/Serializer for Elixir","doc":"All functions return structured errors with context:\n\n```elixir\n{:error, %BTHome.Error{\n  type: :validation,  # :validation, :encoding, or :decoding\n  message: \"Unsupported measurement type: :invalid\",\n  context: %{type: :invalid}  # Additional debugging info\n}}\n```","ref":"readme.html#error-handling"},{"type":"extras","title":"Advanced Usage - BTHome v2 Parser/Serializer for Elixir","doc":"","ref":"readme.html#advanced-usage"},{"type":"extras","title":"Custom Object IDs - BTHome v2 Parser/Serializer for Elixir","doc":"```elixir\n# Override default object ID (advanced use case)\n{:ok, measurement} = BTHome.measurement(:temperature, 23.45, object_id: 0x02)\n```","ref":"readme.html#custom-object-ids"},{"type":"extras","title":"Backwards Compatibility - BTHome v2 Parser/Serializer for Elixir","doc":"The library maintains backwards compatibility with the legacy API:\n\n```elixir\nBTHome.serialize(measurements)\nBTHome.deserialize(binary)\nBTHome.measurement(:temperature, 23.45)\n```","ref":"readme.html#backwards-compatibility"},{"type":"extras","title":"Performance Considerations - BTHome v2 Parser/Serializer for Elixir","doc":"The library uses compile-time optimizations for maximum performance:\n\n- O(1) type lookups using compile-time maps\n- Set-based binary sensor detection\n- Minimal runtime overhead for validation","ref":"readme.html#performance-considerations"},{"type":"extras","title":"Error Recovery - BTHome v2 Parser/Serializer for Elixir","doc":"The decoder includes error recovery for unknown object IDs:\n\n```elixir\n# If binary contains unknown object IDs, they are skipped\n# and parsing continues with known measurements\n{:ok, decoded} = BTHome.deserialize(binary_with_unknown_data)\n# Successfully returns known measurements, skips unknown ones\n```","ref":"readme.html#error-recovery"},{"type":"extras","title":"Examples - BTHome v2 Parser/Serializer for Elixir","doc":"","ref":"readme.html#examples"},{"type":"extras","title":"IoT Sensor Data - BTHome v2 Parser/Serializer for Elixir","doc":"```elixir\n# Traditional approach\nmeasurements = [\n  %{type: :temperature, value: 22.5},\n  %{type: :humidity, value: 45.0},\n  %{type: :battery, value: 85}\n]\n\n{:ok, binary} = BTHome.serialize(measurements)\ndata = BTHome.deserialize_measurements!(binary)\n# => %{temperature: 22.5, humidity: 45.0, battery: 85}\n\n# IoT Sensor Data - Builder pattern approach\nbinary = BTHome.Packet.new()\n|> BTHome.Packet.add_measurement(:temperature, 22.5)\n|> BTHome.Packet.add_measurement(:humidity, 45.0)\n|> BTHome.Packet.add_measurement(:battery, 85)\n|> BTHome.Packet.serialize!()\n\ndata = BTHome.deserialize_measurements!(binary)\n# => %{temperature: 22.5, humidity: 45.0, battery: 85}\n```","ref":"readme.html#iot-sensor-data"},{"type":"extras","title":"Home Automation - BTHome v2 Parser/Serializer for Elixir","doc":"```elixir\n# Traditional approach\nmeasurements = [\n  %{type: :motion, value: true},\n  %{type: :door, value: false},\n  %{type: :temperature, value: 21.0}\n]\n\n{:ok, binary} = BTHome.serialize(measurements)\ndata = BTHome.deserialize_measurements!(binary)\n# => %{motion: true, door: false, temperature: 21.0}\n\n# Home Automation - Builder pattern approach\nbinary = BTHome.Packet.new()\n|> BTHome.Packet.add_measurement(:motion, true)\n|> BTHome.Packet.add_measurement(:door, false)\n|> BTHome.Packet.add_measurement(:temperature, 21.0)\n|> BTHome.Packet.serialize!()\n\ndata = BTHome.deserialize_measurements!(binary)\n# => %{motion: true, door: false, temperature: 21.0}\n```","ref":"readme.html#home-automation"},{"type":"extras","title":"Environmental Monitoring - BTHome v2 Parser/Serializer for Elixir","doc":"```elixir\n# Environmental Monitoring - Traditional approach\nmeasurements = [\n  %{type: :temperature, value: 23.1},\n  %{type: :humidity, value: 58.3},\n  %{type: :pressure, value: 1013.25},\n  %{type: :pm2_5, value: 12},\n  %{type: :pm10, value: 18},\n  %{type: :co2, value: 420}\n]\n\n{:ok, binary} = BTHome.serialize(measurements)\ndata = BTHome.deserialize_measurements!(binary)\n# => %{temperature: 23.1, humidity: 58.3, pressure: 1013.25, pm2_5: 12, pm10: 18, co2: 420}\n\n# Environmental Monitoring - Builder pattern approach\nbinary = BTHome.Packet.new()\n|> BTHome.Packet.add_measurement(:temperature, 23.1)\n|> BTHome.Packet.add_measurement(:humidity, 58.3)\n|> BTHome.Packet.add_measurement(:pressure, 1013.25)\n|> BTHome.Packet.add_measurement(:pm2_5, 12)\n|> BTHome.Packet.add_measurement(:pm10, 18)\n|> BTHome.Packet.add_measurement(:co2, 420)\n|> BTHome.Packet.serialize!()\n\ndata = BTHome.deserialize_measurements!(binary)\n# => %{temperature: 23.1, humidity: 58.3, pressure: 1013.25, pm2_5: 12, pm10: 18, co2: 420}\n```","ref":"readme.html#environmental-monitoring"},{"type":"extras","title":"Testing - BTHome v2 Parser/Serializer for Elixir","doc":"Run the test suite:\n\n```bash\nmix test\n```\n\nThe library includes comprehensive tests covering:\n- All sensor types and their ranges\n- Serialization/deserialization round trips\n- Validation edge cases\n- Error recovery scenarios\n- Backwards compatibility","ref":"readme.html#testing"},{"type":"extras","title":"Contributing - BTHome v2 Parser/Serializer for Elixir","doc":"1. Fork the repository\n2. Create a feature branch\n3. Add tests for new functionality\n4. Ensure all tests pass\n5. Submit a pull request","ref":"readme.html#contributing"},{"type":"extras","title":"License - BTHome v2 Parser/Serializer for Elixir","doc":"This project is licensed under the MIT License - see the LICENSE file for details.","ref":"readme.html#license"},{"type":"extras","title":"References - BTHome v2 Parser/Serializer for Elixir","doc":"- [BTHome v2 Specification](https://bthome.io/)\n- [Bluetooth Low Energy](https://www.bluetooth.com/specifications/bluetooth-core-specification/)\n\nDocumentation can be generated with [ExDoc](https://github.com/elixir-lang/ex_doc)\nand published on [HexDocs](https://hexdocs.pm). Once published, the docs can\nbe found at  .","ref":"readme.html#references"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.2"}}